
#ifndef _CYRIL_IMAGE_HPP_
#define _CYRIL_IMAGE_HPP_

#include "cyril-macro.h"

// open cv
#ifdef _OPENCV_
	#include <cv.h>
	#include <highgui.h>
#endif
#include <iostream>
#include <sstream>

namespace cyril {

template<typename T>
class Image
{
 public:
	const int depth; /// depth in bytes
	T *data_; /// pointer to raw data_
	int w, h; /// size in pixels
	int chan; /// number of chan
	int step; /// width step
	
	static int id;
	
	// constructors and desctructors
	Image() : depth(sizeof(T)), data_(NULL)
		{ id++; }
	Image(Image<T> &img) : depth(sizeof(T)), data_(NULL)
		{ create(img.w,img.h,img.chan,img.step); for(int i=0; i<img.step*img.h; i++) data_[i]=img.data[i]; id++; }
	Image(int width, int height, int channels, int widthStep=-1) : depth(sizeof(T)), data_(NULL)
		{ create(width, height, channels, widthStep); id++; }
	Image(T *data_, int width, int height, int channels, int widthStep=-1) : depth(sizeof(T)), data_(NULL)
		{ wrap(data_, width, height, channels, widthStep); id++; }
	~Image() 
		{ destroy(); }
	
	inline void set(int width, int height, int channels, int widthStep=-1) 
		{ w=width; h=height; chan=channels; if (widthStep < 0) step=w*depth*chan; else step = widthStep; }
	inline void create(int width, int height, int channels, int widthStep=-1) 
		{ set(width, height, channels, widthStep); data_ = (T*) new uchar[step*h]; }
	inline void wrap(T *data_, int width, int height, int channels, int widthStep=-1)
		{ delete[] data_; set(width, height, channels, widthStep); data_ = data_; if (widthStep >= 0) step = widthStep; }
	inline void destroy() 
		{ delete[] data_; data_ = NULL; }
	inline void clear() 
		{ data_ = NULL; }
	
	// access
//	inline T& operator[](int y, int x, int c=0) 
//		{ return data_[y*step+x*chan+c]; }
	inline T& data(int y, int x, int c=0) 
		{ return data_[y*step+x*chan+c]; }
	inline T& operator()(int y, int x, int c=0) 
		{ if (x<0) x=0; if (y<0) y=0; if (x>=w) x=w-1; if (y>=h) y=h-1; return data_[y*step+x*chan+c]; }

	// IOs
	bool load(const char *name, bool grey = false); /// load image from the file "name"
	bool save(const char *name); /// save image to the file "name"
	bool show(const char *name); /// show image in an existing window
	void show(); /// show image in a new window that will be closed right after
	
	static void openWindow(const char *name);
	static void closeWindow(const char *name);
	
	// operations
	enum Mask { gradC, gradU, gradB, sobel };
	inline real gradV(int y, int x, Mask mask = gradC, bool table = false, bool check = false);
	inline real gradH(int y, int x, Mask mask = gradC, bool table = false, bool check = false);
	inline real gradN2(int y, int x, Mask mask = gradC, bool table = false, bool check = false);
	inline real gradN(int y, int x, Mask mask = gradC, bool table = false, bool check = false);
	
 protected:
	bool save_show(const char *name, bool save);
};

#include "cyril-image.cxx"


template<typename T>
real Image<T>::gradV(int y, int x, Mask mask, bool table, bool check)
{
	switch (mask)
	{
		case gradC:
			return (data(y+1, x) - data(y-1, x)) / 2.;
		default:
			return 0;
	}

}


template<typename T>
real Image<T>::gradH(int y, int x, Mask mask, bool table, bool check)
{
	switch (mask)
	{
		case gradC:
			return (data(y, x+1) - data(y, x-1)) / 2.;
		default:
			return 0;
	}

}

template<typename T>
real Image<T>::gradN2(int y, int x, Mask mask, bool table, bool check)
{
	return sqr(gradV(y,x,mask,table,check)) + sqr(gradH(y,x,mask,table,check));
}

template<typename T>
real Image<T>::gradN(int y, int x, Mask mask, bool table, bool check)
{
	return sqrt(gradN2);
}

















} // cyril


#endif

